@@ -1,11 +1,12 @@
 from .inference import (inference_bottom_up_pose_model,
                         inference_top_down_pose_model, init_pose_model,
                         vis_pose_result)
+from .inference_tracking import get_track_id, vis_pose_tracking_result
 from .test import multi_gpu_test, single_gpu_test
 from .train import train_model
 
 __all__ = [
     'train_model', 'init_pose_model', 'inference_top_down_pose_model',
     'inference_bottom_up_pose_model', 'multi_gpu_test', 'single_gpu_test',
-    'vis_pose_result'
+    'vis_pose_result', 'get_track_id', 'vis_pose_tracking_result'
 ]
