@@ -191,6 +191,12 @@ def _inference_single_pose_model(model,
         flip_pairs = [[0, 3], [1, 4], [2, 5], [6, 9], [7, 10], [8, 11]]
     elif dataset == 'TopDownMpiiDataset':
         flip_pairs = [[0, 5], [1, 4], [2, 3], [10, 15], [11, 14], [12, 13]]
+    elif dataset == 'TopDownMpiiTrbDataset':
+        flip_pairs = [[0, 1], [2, 3], [4, 5], [6, 7],
+                      [8, 9], [10, 11], [14, 15], [16, 22], [28, 34], [17, 23],
+                      [29, 35], [18, 24], [30, 36], [19, 25], [31,
+                                                               37], [20, 26],
+                      [32, 38], [21, 27], [33, 39]]
     elif dataset in ('OneHand10KDataset', 'FreiHandDataset', 'PanopticDataset',
                      'InterHand2DDataset'):
         flip_pairs = []
@@ -499,6 +505,16 @@ def vis_pose_result(model,
             16, 16, 16, 16, 16, 16, 7, 7, 0, 0, 9, 9, 9, 9, 9, 9
         ]]
 
+    elif dataset == 'TopDownMpiiTrbDataset':
+        skeleton = [[13, 14], [14, 1], [14, 2], [1, 3], [2, 4], [3, 5], [4, 6],
+                    [1, 7], [2, 8], [7, 8], [7, 9], [8, 10], [9, 11], [10, 12],
+                    [15, 16], [17, 18], [19, 20], [21, 22], [23, 24], [25, 26],
+                    [27, 28], [29, 30], [31, 32], [33, 34], [35, 36], [37, 38],
+                    [39, 40]]
+
+        pose_limb_color = palette[[16] * 14 + [19] * 13]
+        pose_kpt_color = palette[[16] * 14 + [0] * 26]
+
     elif dataset in ('OneHand10KDataset', 'FreiHandDataset',
                      'PanopticDataset'):
         skeleton = [[1, 2], [2, 3], [3, 4], [4, 5], [1, 6], [6, 7], [7, 8],
