@@ -3,11 +3,12 @@ from .bottom_up_eval import (aggregate_results, get_group_preds,
 from .eval_hooks import DistEvalHook, EvalHook
 from .mesh_eval import compute_similarity_transform
 from .top_down_eval import (keypoint_auc, keypoint_epe, keypoint_pck_accuracy,
-                            keypoints_from_heatmaps, pose_pck_accuracy)
+                            keypoints_from_heatmaps, pose_pck_accuracy,
+                            post_dark_udp)
 
 __all__ = [
     'EvalHook', 'DistEvalHook', 'pose_pck_accuracy', 'keypoints_from_heatmaps',
     'keypoint_pck_accuracy', 'keypoint_auc', 'keypoint_epe', 'get_group_preds',
     'get_multi_stage_outputs', 'aggregate_results',
-    'compute_similarity_transform'
+    'compute_similarity_transform', 'post_dark_udp'
 ]
