@@ -1,4 +1,5 @@
 import os
+import warnings
 from collections import OrderedDict, defaultdict
 
 import json_tricks as json
@@ -63,13 +64,17 @@ class TopDownCocoDataset(TopDownBaseDataset):
 
         self.use_gt_bbox = data_cfg['use_gt_bbox']
         self.bbox_file = data_cfg['bbox_file']
-        self.image_thr = data_cfg['image_thr']
+        self.det_bbox_thr = data_cfg.get('det_bbox_thr', 0.0)
+        if 'image_thr' in data_cfg:
+            warnings.warn(
+                'image_thr is deprecated, '
+                'please use det_bbox_thr instead', DeprecationWarning)
+            self.det_bbox_thr = data_cfg['image_thr']
         self.use_nms = data_cfg.get('use_nms', True)
         self.soft_nms = data_cfg['soft_nms']
         self.nms_thr = data_cfg['nms_thr']
         self.oks_thr = data_cfg['oks_thr']
         self.vis_thr = data_cfg['vis_thr']
-        self.bbox_thr = data_cfg['bbox_thr']
 
         self.ann_info['flip_pairs'] = [[1, 2], [3, 4], [5, 6], [7, 8], [9, 10],
                                        [11, 12], [13, 14], [15, 16]]
@@ -183,6 +188,7 @@ class TopDownCocoDataset(TopDownBaseDataset):
                 valid_objs.append(obj)
         objs = valid_objs
 
+        bbox_id = 0
         rec = []
         for obj in objs:
             if 'keypoints' not in obj:
@@ -210,8 +216,10 @@ class TopDownCocoDataset(TopDownBaseDataset):
                 'joints_3d': joints_3d,
                 'joints_3d_visible': joints_3d_visible,
                 'dataset': self.dataset_name,
-                'bbox_score': 1
+                'bbox_score': 1,
+                'bbox_id': bbox_id
             })
+            bbox_id = bbox_id + 1
 
         return rec
 
@@ -259,7 +267,7 @@ class TopDownCocoDataset(TopDownBaseDataset):
         print(f'=> Total boxes: {len(all_boxes)}')
 
         kpt_db = []
-        num_boxes = 0
+        bbox_id = 0
         for det_res in all_boxes:
             if det_res['category_id'] != 1:
                 continue
@@ -269,11 +277,9 @@ class TopDownCocoDataset(TopDownBaseDataset):
             box = det_res['bbox']
             score = det_res['score']
 
-            if score < self.image_thr:
+            if score < self.det_bbox_thr:
                 continue
 
-            num_boxes = num_boxes + 1
-
             center, scale = self._xywh2cs(*box[:4])
             joints_3d = np.zeros((num_joints, 3), dtype=np.float32)
             joints_3d_visible = np.ones((num_joints, 3), dtype=np.float32)
@@ -286,10 +292,12 @@ class TopDownCocoDataset(TopDownBaseDataset):
                 'bbox_score': score,
                 'dataset': self.dataset_name,
                 'joints_3d': joints_3d,
-                'joints_3d_visible': joints_3d_visible
+                'joints_3d_visible': joints_3d_visible,
+                'bbox_id': bbox_id
             })
+            bbox_id = bbox_id + 1
         print(f'=> Total boxes after filter '
-              f'low score@{self.image_thr}: {num_boxes}')
+              f'low score@{self.det_bbox_thr}: {bbox_id}')
         return kpt_db
 
     def evaluate(self, outputs, res_folder, metric='mAP', **kwargs):
@@ -308,10 +316,10 @@ class TopDownCocoDataset(TopDownBaseDataset):
                     coordinates, score is the third dimension of the array.
                 :boxes (np.ndarray[1,6]): [center[0], center[1], scale[0]
                     , scale[1],area, score]
-                :image_path (list[str]): For example, [ '/', 'v','a', 'l',
-                    '2', '0', '1', '7', '/', '0', '0', '0', '0', '0',
-                    '0', '3', '9', '7', '1', '3', '3', '.', 'j', 'p', 'g']
-                :heatmap (np.ndarray[N, K, H, W]): model output heatmap.
+                :image_path (list[str]): For example, ['data/coco/val2017
+                    /000000393226.jpg']
+                :heatmap (np.ndarray[N, K, H, W]): model output heatmap
+                :bbox_id (list(int)).
             res_folder (str): Path of directory to save the results.
             metric (str | list[str]): Metric to be performed. Defaults: 'mAP'.
 
@@ -327,18 +335,20 @@ class TopDownCocoDataset(TopDownBaseDataset):
         res_file = os.path.join(res_folder, 'result_keypoints.json')
 
         kpts = defaultdict(list)
-        for preds, boxes, image_path, _ in outputs:
-            str_image_path = ''.join(image_path)
-            image_id = self.name2id[str_image_path[len(self.img_prefix):]]
-
-            kpts[image_id].append({
-                'keypoints': preds[0],
-                'center': boxes[0][0:2],
-                'scale': boxes[0][2:4],
-                'area': boxes[0][4],
-                'score': boxes[0][5],
-                'image_id': image_id,
-            })
+        for preds, boxes, image_paths, _, bbox_ids in outputs:
+            batch_size = len(image_paths)
+            for i in range(batch_size):
+                image_id = self.name2id[image_paths[i][len(self.img_prefix):]]
+                kpts[image_id].append({
+                    'keypoints': preds[i],
+                    'center': boxes[i][0:2],
+                    'scale': boxes[i][2:4],
+                    'area': boxes[i][4],
+                    'score': boxes[i][5],
+                    'image_id': image_id,
+                    'bbox_id': bbox_ids[i]
+                })
+        kpts = self._sort_and_unique_bboxes(kpts)
 
         # rescoring and oks nms
         num_joints = self.ann_info['num_joints']
@@ -436,3 +446,14 @@ class TopDownCocoDataset(TopDownBaseDataset):
         info_str = list(zip(stats_names, coco_eval.stats))
 
         return info_str
+
+    def _sort_and_unique_bboxes(self, kpts, key='bbox_id'):
+        """sort kpts and remove the repeated ones."""
+        for img_id, persons in kpts.items():
+            num = len(persons)
+            kpts[img_id] = sorted(kpts[img_id], key=lambda x: x[key])
+            for i in range(num - 1, 0, -1):
+                if kpts[img_id][i][key] == kpts[img_id][i - 1][key]:
+                    del kpts[img_id][i]
+
+        return kpts
