@@ -13,9 +13,6 @@ class BaseBackbone(nn.Module, metaclass=ABCMeta):
     inherits this class should at least define its own `forward` function.
     """
 
-    def __init__(self):
-        super().__init__()
-
     def init_weights(self, pretrained=None):
         """Init backbone weights.
 
@@ -43,12 +40,3 @@ class BaseBackbone(nn.Module, metaclass=ABCMeta):
             x (tensor | tuple[tensor]): x could be a Torch.tensor or a tuple of
                 Torch.tensor, containing input data for forward computation.
         """
-        pass
-
-    def train(self, mode=True):
-        """Set module status before forward computation.
-
-        Args:
-            mode (bool): Whether it is train_mode or test_mode
-        """
-        super().train(mode)
