@@ -1,7 +1,8 @@
+from .mesh_loss import GANLoss, MeshLoss
 from .mse_loss import JointsMSELoss, JointsOHKMMSELoss
 from .multi_loss_factory import AELoss, HeatmapLoss, MultiLossFactory
 
 __all__ = [
     'JointsMSELoss', 'JointsOHKMMSELoss', 'HeatmapLoss', 'AELoss',
-    'MultiLossFactory'
+    'MultiLossFactory', 'MeshLoss', 'GANLoss'
 ]
